/**************************************************************************//**
*   @file   ad5668.h
*   @brief  Header file of AD5668 Driver for NIOS processor.
*   @author grohmast
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 158
******************************************************************************/
#ifndef _AD5668_H_
#define _AD5668_H_
#ifdef AD5668_GLOBALS
	#define AD5668_EXT extern volatile
#else
	#define AD5668_EXT volatile
#endif

/****************************************************************************/
/*  Include Files                                                           */
/****************************************************************************/
#include "adi.h"
#if (USE_AD5668 == 1)

/****************************************************************************/
/*  AD5668                                                                  */
/****************************************************************************/
typedef enum _AD5668Command
{
	WRITE_TO_INPUT_REGISTER_N 					= 0x00,
	UPDATE_DAC_REGISTER_N 						= 0x01,
	WRITE_TO_INPUT_REGISTER_N_UPDATE_ALL 		= 0x02,
	WRITE_TO_REGISTER_AND_UPDATE_CHANNEL_N		= 0x03,
	POWER_DOWN_POWER_UP_DAC						= 0x04,
	LOAD_CLEAR_CODE_REGISTER					= 0x05,
	LOAD_LDAC_REGISTER							= 0x06,
	RESET										= 0x07,
	SET_UP_INTERNAL_REF_REGISTER				= 0x08,
}AD5668Command;
typedef enum _AD5668Channel
{
	CHANNEL_A	= 0,
	CHANNEL_B	= 1,
	CHANNEL_C	= 2,
	CHANNEL_D	= 3,
	CHANNEL_E	= 4,
	CHANNEL_F	= 5,
	CHANNEL_G	= 6,
	CHANNEL_H	= 7,
	CHANNEL_ALL	= 0xF,
}AD5668Channel;
#define AD5668CMDPOSITION		24		//used for shifting the command bits
#define AD5668CHANPOSITION		20		//used for shifting the channel bits
#define AD5668DATAPOSITION		4		//used for shifting the data bits
#define INTERN_REF_ON			(1)
#define PWR_NORM				(0)
#define PWR_1K					(1)
#define PWR_100K				(2)
#define PWR_THREES				(3)
#define AD5668PWRPOSITION		8

/****************************************************************************/
/*  GPIO                                                                    */
/****************************************************************************/
#define GPIO_LDAC_N				(1<<0)
#define GPIO_DAC_CLR_N			(1<<1)
#define GPIO_DAC				(1<<2)
#define GPIO_ADC				(1<<3)
#define GPIO_MUX				(1<<4)
#define GIO_DIR_OUT				1
#define GIO_DIR_IN				0
#define MUX_TARGET_DAC			0
#define MUX_TARGET_ADC			1
#define MUX_TARGET_MUX			2

/****************************************************************************/
/*  ADG738 (MUX) 8bit                                                       */
/****************************************************************************/
typedef enum _ADG738Channel
{
	ADG738_CHANNEL_A	= (1<<0),
	ADG738_CHANNEL_B	= (1<<1),
	ADG738_CHANNEL_C	= (1<<2),
	ADG738_CHANNEL_D	= (1<<3),
	ADG738_CHANNEL_E	= (1<<4),
	ADG738_CHANNEL_F	= (1<<5),
	ADG738_CHANNEL_G	= (1<<6),
	ADG738_CHANNEL_H	= (1<<7),
}ADG738Channel;

/****************************************************************************/
/*  AD7475 (ADC) 16bit                                                      */
/****************************************************************************/
#define ad5668_ADC_DATAMASK	0x0FFF

/****************************************************************************/
/* SPI Configuration                                                        */
/****************************************************************************/
#define AD5668_SPI			SPI_1_P0_BASE
#define AD5668_SPI_CFG		(SPI1_ena|SPI1_inv)

/****************************************************************************/
/* Monitored variables declarations                                         */
/****************************************************************************/
AD5668_EXT uCProbeDataUnion AD5668_Data[14];	//used for the interface with uC/Probe
AD5668_EXT unsigned short rgADCValue[8];		//used to store data read by ADC
AD5668_EXT float rgVRead[8];					//used for displaying the voltages read with the ADC
#endif //(USE_AD5668 == 1)

/****************************************************************************/
/* Functions Prototypes                                                     */
/****************************************************************************/
int AD5668_Init(void);								//Initialize the AD5668 evaluation board
int AD5668_UpdateParameters(void);					//Update the device's monitored and controlled parameters.
int AD5668_GetAd7475(void);							//Get OnBoard ADC data
int AD5668_SetAdg738(int iValue);			    	//Set OnBoard MUX
int AD5668_Set(int iValue);							//Set OnBoard DAC data
int AD5668_GetAdc(int channel);						//Read voltages on different channels
int AD5668_SetDacCommand(char command, char channel,
						 unsigned short iDAC);		//Function used to send a specific command to the AD5668
void AD5668_Clear(void);							//Toggle clear data pin
void AD5668_Mux(int target);						//Used to set the communication MUX
void AD5668_Ldac(void);								//Toggle LDAC pin
void AD5668_SetPor(void);							//Initialize all channels with power mode normal
void AD5668_SetRef(void);							//Set the internal reference voltage on
void AD5668_SetDac(int channel, int iDAC);			//Used to set the out value on the DAC
#endif /* _AD5668_H_ */
